/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2012, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.samples.wsse.policy.basic;

import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.soap.SOAPFaultException;

import org.apache.cxf.Bus;
import org.apache.cxf.BusFactory;
import org.apache.cxf.ws.security.SecurityConstants;
import org.jboss.ws.api.configuration.ClientConfigUtil;
import org.jboss.wsf.test.ClientHelper;
import org.jboss.wsf.test.CryptoHelper;

public class SignEncryptHelper implements ClientHelper
{
   private String targetEndpoint;
   
   @Override
   public void setTargetEndpoint(String address)
   {
      targetEndpoint = address;
   }
   
   public boolean testSignEncrypt() throws Exception
   {
      Bus bus = BusFactory.newInstance().createBus();
      try
      {
         BusFactory.setThreadDefaultBus(bus);
         ServiceIface proxy = getProxy();
         setupWsse(proxy);
         return invoke(proxy);
      }
      finally
      {
         bus.shutdown(true);
      }
   }
   
   public boolean testSignEncryptUsingConfigProperties() throws Exception
   {
      Bus bus = BusFactory.newInstance().createBus();
      try
      {
         BusFactory.setThreadDefaultBus(bus);
         ServiceIface proxy = getProxy();
         ClientConfigUtil.setConfigProperties(proxy, "META-INF/jaxws-client-config.xml", "Custom WS-Security Client");
         return invoke(proxy);
      }
      finally
      {
         bus.shutdown(true);
      }
   }
   
   private ServiceIface getProxy() throws MalformedURLException {
      QName serviceName = new QName("http://www.jboss.org/jbossws/ws-extensions/wssecuritypolicy", "SecurityService");
      URL wsdlURL = new URL(targetEndpoint + "?wsdl");
      Service service = Service.create(wsdlURL, serviceName);
      return (ServiceIface)service.getPort(ServiceIface.class);
   }
   
   private boolean invoke(ServiceIface proxy) throws Exception {
      try
      {
         return "Secure Hello World!".equals(proxy.sayHello());
      }
      catch (SOAPFaultException e)
      {
         throw CryptoHelper.checkAndWrapException(e);
      }
   }
   
   private void setupWsse(ServiceIface proxy)
   {
      ((BindingProvider)proxy).getRequestContext().put(SecurityConstants.CALLBACK_HANDLER, new KeystorePasswordCallback());
      ((BindingProvider)proxy).getRequestContext().put(SecurityConstants.SIGNATURE_PROPERTIES, Thread.currentThread().getContextClassLoader().getResource("META-INF/alice.properties"));
      ((BindingProvider)proxy).getRequestContext().put(SecurityConstants.ENCRYPT_PROPERTIES, Thread.currentThread().getContextClassLoader().getResource("META-INF/alice.properties"));
      ((BindingProvider)proxy).getRequestContext().put(SecurityConstants.SIGNATURE_USERNAME, "alice");
      ((BindingProvider)proxy).getRequestContext().put(SecurityConstants.ENCRYPT_USERNAME, "bob");
   }
}
